#-*- perl -*-
#-*- coding: us-ascii -*-

package Encode::ISOIRSingle;

use strict;
use warnings;
our $VERSION = '0.01';

use XSLoader;
XSLoader::load(__PACKAGE__, $VERSION);

1;
__END__

=head1 NAME

Encode::ISOIRSingle - ISO-IR single byte coded charcter sets

=head1 DESCRIPTION

See L<Encode::ISO2022::CCS>.

=cut
